@@ -321,10 +321,8 @@ fr_command_7z_add (FrCommand  *command,
 	else
 		fr_process_add_arg (command->process, "a");
 
-	if (base_dir != NULL) {
+	if (base_dir != NULL)
 		fr_process_set_working_dir (command->process, base_dir);
-		fr_process_add_arg_concat (command->process, "-w", base_dir, NULL);
-	}
 
 	if (_g_mime_type_matches (archive->mime_type, "application/zip")
 	    || _g_mime_type_matches (archive->mime_type, "application/x-cbz"))
@@ -392,7 +390,8 @@ fr_command_7z_delete (FrCommand  *command,
 		      const char *from_file,
 		      GList      *file_list)
 {
-	GList *scan;
+	FrArchive *archive = FR_ARCHIVE (command);
+	GList     *scan;
 
 	fr_command_7z_begin_command (command);
 	fr_process_add_arg (command->process, "d");
@@ -401,6 +400,12 @@ fr_command_7z_delete (FrCommand  *command,
 	if (_g_mime_type_matches (FR_ARCHIVE (command)->mime_type, "application/x-ms-dos-executable"))
 		fr_process_add_arg (command->process, "-sfx");
 
+	if (_g_mime_type_matches (archive->mime_type, "application/zip")
+	    || _g_mime_type_matches (archive->mime_type, "application/x-cbz"))
+	{
+		fr_process_add_arg (command->process, "-tzip");
+	}
+
 	if (from_file != NULL)
 		fr_process_add_arg_concat (command->process, "-i@", from_file, NULL);
 
@@ -674,6 +679,7 @@ fr_command_7z_init (FrCommand7z *self)
 	base->propAddCanReplace            = TRUE;
 	base->propAddCanStoreFolders       = TRUE;
 	base->propAddCanStoreLinks         = TRUE;
+	base->propAddCanFollowDirectoryLinksWithoutDereferencing = FALSE;
 	base->propExtractCanAvoidOverwrite = FALSE;
 	base->propExtractCanSkipOlder      = FALSE;
 	base->propExtractCanJunkPaths      = TRUE;
